/**
 @class GrassIkComp
 @author YI ZHANG
 @date 2023/4/21
 @desc
 **/

import * as cc from "cc";

const {ccclass, property} = cc._decorator;

export interface GrassIkCompData {
}

@ccclass("GrassIkComp")
export default class GrassIkComp extends cc.Component {
    v3 = cc.v3();
    v3_2 = cc.v3();
    private startPos !: cc.Vec3;
    body !: cc.RigidBody2D;
    spine: cc.sp.Skeleton = null!;
    @property(cc.CCString)
    boneName = "";
    private isRecoveryPos: boolean = false;

    localToWorld = function (this : cc.sp.spine.Bone, local : cc.Vec2 | cc.Vec3) {
        var x = local.x, y = local.y;
        local.x = x * this.a + y * this.b + this.worldX;
        local.y = x * this.c + y * this.d + this.worldY;
        return local;
    };

    worldToLocal = function (this : cc.sp.spine.Bone,world : cc.Vec2 | cc.Vec3) {
        var a = this.a, b = this.b, c = this.c, d = this.d;
        var invDet = 1 / (a * d - b * c);
        var x = world.x - this.worldX, y = world.y - this.worldY;
        world.x = (x * d * invDet - y * b * invDet);
        world.y = (y * a * invDet - x * c * invDet);
        return world;
    };

    init(spine : cc.sp.Skeleton,name : string,size : number,isRecoveryPos : boolean,gravityScale : number){
        this.boneName = name;
        this.spine = spine;
        let bone = this.spine.findBone(this.boneName);
        if (bone) {
            bone.updateWorldTransform();
            let pos = this.localToWorld.call(bone,cc.v2(0,0))
            this.node.setPosition(pos.x,pos.y);
        }

        this.isRecoveryPos = isRecoveryPos;
        this.startPos = this.node.getPosition();
        return this;
    }

    connectBody(body : cc.RigidBody2D){
        let joint = this.addComponent(cc.DistanceJoint2D)!;
        joint.connectedBody = body;
        joint.autoCalcDistance = true;
        joint.collideConnected = true;
    }

    protected lateUpdate(dt: number) {
        if(!this.isRecoveryPos){
            this.node.getPosition(this.v3_2);
            let bone = this.spine.findBone(this.boneName);
            if (bone) {
                let pos = this.worldToLocal.call(bone.parent,this.v3_2);
                bone.x = pos.x;
                bone.y = pos.y;
            }
            return;
        }
        this.node.getPosition(this.v3_2);
        let bone = this.spine.findBone(this.boneName);
        if (bone) {
            let pos = this.worldToLocal.call(bone.parent,this.v3_2);
            bone.x = pos.x;
            bone.y = pos.y;
        }

    }
}

